-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  AdcReceiver
-- File Name:    AdcReceiver.vhd
-- File Path:    Projects\Ads527x\Vhdl\Virtex2V1000Platform\AdcReceiver
-- Project : 
--
-- Purpose:     Toplevel of the ADC receiver interface in the FPGA.
--                              Contains the RECEIVER logic (input shifter and register) and
--                              the TIMING interface.
--
--                              Specially adapted for the Texas Instruments XC2V1000 Test Board.
--                              Only one channel per BRAM.
--
-- Limitations: none
--
-- Authors: Marc Defossez
--
-- Tools: 
--
--  Contact:    e-mail  hotline@xilinx.com        phone   + 1 800 255 7778
--
--  Disclaimer:
--              LIMITED WARRANTY AND DISCLAIMER.
--              These designs are provided to you "as is". Xilinx and its licensors make and you
--              receive no warranties or conditions, express, implied, statutory or otherwise, and
--              Xilinx specifically disclaims any implied warranties of merchantability, non-infringement,
--              or fitness for a particular purpose. Xilinx does not warrant that the functions contained
--              in these designs will meet your requirements, or that the operation of these designs will
--              be uninterrupted or error free, or that defects in the designs will be corrected.
--              Furthermore, Xilinx does not warrant or make any representations regarding use or the
--              results of the use of the designs in terms of correctness, accuracy, reliability, or otherwise.
--
--              LIMITATION OF LIABILITY.
--              In no event will Xilinx or its licensors be liable for any loss of data, lost profits, cost or
--              procurement of substitute goods or services, or for any special, incidental, consequential,
--              or indirect damages arising from the use or operation of the designs or accompanying
--              documentation, however caused and on any theory of liability. This limitation will apply
--              even if Xilinx has been advised of the possibility of such damage. This limitation shall
--              apply not-withstanding the failure of the essential purpose of any limited remedies herein.
--
--              Copyright  2002 Xilinx, Inc.
--              All rights reserved
--
-- Revision History:
--                      Created:        20/04/04
--                      Modified        22/06/04
--                              Added the possibility to use a Block SelectRAM as register.
--                              Generic value is used to select his possibility.
--                              When "BramAsRegister" generic value is:
--                                      1 use BRAM as register.
--                                      0 straight output from this source.
--                              Signals RXEnaRam and RxClkRam will be GND-ed when BRAM is used as register.
--
----------------------------------------------------------------------------------------------------------------------------------------
--
LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
-- Simulation libraries
-- synopsys translate_off
LIBRARY UNISIM;
USE UNISIM.ALL;
-- synopsys translate_on
-- end simulation libraries

-- Libraries for Synplicity
-- Needed for components as LUT, SRL which get an INIT parameter.
-- When this library is not mentioned Synplify_Pro fails during synthesis of
-- the hardware with an error about the INIT value.
-- library virtex2;
-- use virtex2.components.all;
-- This library is ignored by Mentor Precision.
-- Only a warning is generated.
-- End libraries for Synplicity

ENTITY AdcReceiver IS
    GENERIC (
        BramAsRegister : INTEGER := 1;  -- 1 use a BRAM as register, 0 don't.
        FfAsRegister   : INTEGER := 0   -- 1 use FFs as register, 0 don't.
                                        -- FfAsRegister  BramAsRegister
                                       --                      0               0               No register at all, direct out.
                                       --                      0               1               Block SelectRAM used as register.
                                       --                      1               0               Normal FFs register (look out for extra BUFG's).
                                       --                      1               1               DO NOT USE !            
        );
    PORT (
        RxClk        : IN  STD_LOGIC;
        RxClkNot     : IN  STD_LOGIC;
        RxRst        : IN  STD_LOGIC;
        RxDatIn      : IN  STD_LOGIC;
        RxSyncIn     : IN  STD_LOGIC;
        PreSyncInP   : IN  STD_LOGIC;  -- Sync coming from previous module. if not used tie it to GND.
        PreSyncInN   : IN  STD_LOGIC;  -- Sync coming from previous module. if not used tie it to GND.
        PostSyncOutP : OUT STD_LOGIC;   -- Sync to the next module.
        PostSyncOutN : OUT STD_LOGIC;   -- Sync to the next module.
        AdClkOut     : OUT STD_LOGIC;  -- One Lclk later than PostSyncOut, for signal analyser purposes
        RxEnaRam     : OUT STD_LOGIC;   -- Used with an external BRAM fifo.
        RxDatOut     : OUT STD_LOGIC_VECTOR (11 DOWNTO 0)
        );
END AdcReceiver;

ARCHITECTURE AdcReceiver_arch OF AdcReceiver IS
----------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
----------------------------------------------------------------------------------------------------------------------------------------
    COMPONENT RxTiming
        PORT (
            RxClk        : IN  STD_LOGIC;
            RxClkNot     : IN  STD_LOGIC;
            RxRst        : IN  STD_LOGIC;
            RxSyncIn     : IN  STD_LOGIC;  -- Input from ADCLK clock of ADC
            PreSyncInP   : IN  STD_LOGIC;  -- Sync coming from previous module. if not used tie it to GND.
            PreSyncInN   : IN  STD_LOGIC;  -- Sync coming from previous module. if not used tie it to GND.
            PostSyncOutP : OUT STD_LOGIC;  -- Sync to the next module.
            PostSyncOutN : OUT STD_LOGIC;  -- Sync to the next module.
            AdClkOut     : OUT STD_LOGIC;  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaMuxP    : OUT STD_LOGIC;
            RxEnaMuxN    : OUT STD_LOGIC;
            RxDatMuxEna  : OUT STD_LOGIC;
            RxEnaRam     : OUT STD_LOGIC;  -- This signal normally go to a BRAM Fifo interface
        RxEnaRamN	: out std_logic
            );
    END COMPONENT;

    COMPONENT Receiver
        PORT (
            RxClk       : IN  STD_LOGIC;
            RxClkNot    : IN  STD_LOGIC;
            RxRst       : IN  STD_LOGIC;
            RxDatIn     : IN  STD_LOGIC;
            RxDatOut    : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
            RxEnaMuxP   : IN  STD_LOGIC;
            RxEnaMuxN   : IN  STD_LOGIC;
            RxDatMuxEna : IN  STD_LOGIC
            );
    END COMPONENT;

    COMPONENT RAMB16_S36
        PORT (
            WE   : IN  STD_LOGIC;
            EN   : IN  STD_LOGIC;
            SSR  : IN  STD_LOGIC;
            CLK  : IN  STD_LOGIC;
            ADDR : IN  STD_LOGIC_VECTOR (8 DOWNTO 0);
            DI   : IN  STD_LOGIC_VECTOR (31 DOWNTO 0);
            DIP  : IN  STD_LOGIC_VECTOR (3 DOWNTO 0);
            DOP  : OUT STD_LOGIC_VECTOR (3 DOWNTO 0);
            DO   : OUT STD_LOGIC_VECTOR (31 DOWNTO 0)
            );
    END COMPONENT;
----------------------------------------------------------------------------------------------------------------------------------------
-- Signal Declarations
----------------------------------------------------------------------------------------------------------------------------------------
    SIGNAL IntRxEnaMuxP   : STD_LOGIC;
    SIGNAL IntRxEnaMuxN   : STD_LOGIC;
    SIGNAL IntRxEnaRam    : STD_LOGIC;
    SIGNAL IntRxEnaRamN    : STD_LOGIC;
    SIGNAL IntRxDatOut    : STD_LOGIC_VECTOR (11 DOWNTO 0);
    SIGNAL IntRxRamRegIn  : STD_LOGIC_VECTOR (31 DOWNTO 0);
    SIGNAL IntRxRamRegOut : STD_LOGIC_VECTOR (31 DOWNTO 0);
    SIGNAL IntRxDatMuxEna : STD_LOGIC;

    SIGNAL Low        : STD_LOGIC                     := '0';
    SIGNAL High       : STD_LOGIC                     := '1';
    SIGNAL IntRegAddr : STD_LOGIC_VECTOR (8 DOWNTO 0) := "000000000";
    SIGNAL IntDip     : STD_LOGIC_VECTOR (3 DOWNTO 0) := "0000";

    ATTRIBUTE RLOC            : STRING;
    ATTRIBUTE RLOC OF TheTime : LABEL IS "X0Y3";
    ATTRIBUTE RLOC OF TheRec  : LABEL IS "X0Y0";
-- The Block SelectRAM's are LOC-ed in the UCF file.
----------------------------------------------------------------------------------------------------------------------------------------
--
BEGIN

    TheTime : RxTiming
        PORT MAP (
            RxClk        => RxClk,
            RxClkNot     => RxClkNot,
            RxRst        => RxRst,
            RxSyncIn     => RxSyncIn,
            PreSyncInP   => PreSyncInP,
            PreSyncInN   => PreSyncInN,
            PostSyncOutP => PostSyncOutP,
            PostSyncOutN => PostSyncOutN,
            AdClkOut     => AdClkOut,
            RxEnaMuxP    => IntRxEnaMuxP,
            RxEnaMuxN    => IntRxEnaMuxN,
            RxDatMuxEna  => IntRxDatMuxEna,
            RxEnaRam     => IntRxEnaRam,
        RxEnaRamN	=> IntRxEnaRamN
            );
--
    TheRec : Receiver
        PORT MAP (
            RxClk       => RxClk,
            RxClkNot    => RxClkNot,
            RxRst       => RxRst,
            RxDatIn     => RxDatIn,
            RxDatOut    => IntRxDatOut,
            RxDatMuxEna => IntRxDatMuxEna,
            RxEnaMuxP   => IntRxEnaMuxP,
            RxEnaMuxN   => IntRxEnaMuxN
            );

------------------------------------------------------------------------------------------------------------------    
-- FfAsRegister  BramAsRegister
--                      0               0               No register at all, direct out.
--                      0               1               Block SelectRAM used as register.
--                      1               0               Normal FFs register (look out for extra BUFG's).
--                      1               1               DO NOT USE !
-- Watch also for the RLOC and RLOC_ORIGIN attributes in the different hierarchical levels and UCF file.
--
    NoRegNoBram : IF BramAsRegister = 0 AND FfAsRegister = 0 GENERATE
        RxEnaRam <= IntRxEnaRam;
        RxDatOut <= IntRxDatOut;
    END GENERATE;
--
    BramAsReg : IF BramAsRegister = 1 AND FfAsRegister = 0 GENERATE
-- RLOC or LOC constraints must be set in the UCF file.
    BEGIN

        PROCESS (RxClk, RxRst)
            VARIABLE tmp : STD_LOGIC;
        BEGIN  -- PROCESS
            IF RxRst = '1' THEN         -- asynchronous reset (active high)
                tmp := '0';
            ELSIF RxClk'EVENT AND RxClk = '1' THEN  -- rising clock edge
                IF IntRxEnaRam = '1' THEN
                    tmp := NOT tmp;
                END IF;
                RxEnaRam <= tmp;
            END IF;
        END PROCESS;


--      RxEnaRam <= Low;
        IntRxRamRegIn <= X"00000" & IntRxDatOut;

        TheRamReg : RAMB16_S36
            PORT MAP (
                WE   => High,
                EN   => IntRxEnaRam,
                SSR  => RxRst,
                CLK  => RxClk,
                ADDR => IntRegAddr,
                DI   => IntRxRamRegIn,
                DIP  => IntDip,
                DOP  => OPEN,
                DO   => IntRxRamRegOut
                );
        RxDatOut <= IntRxRamRegOut (11 DOWNTO 0);
    END GENERATE;
--
    FfAsReg : IF BramAsRegister = 0 AND FfAsRegister = 1 GENERATE
-- RLOC or LOC constraints must be set in the UCF file.
    BEGIN
        TheFfReg : PROCESS (RxClk, IntRxEnaRam)
            VARIABLE tmp : STD_LOGIC;
        BEGIN
            IF (RxRst = '1') THEN
--                RxDatOut <= (OTHERS => '0');
                tmp      := '0';
            ELSIF (RxClk'EVENT AND RxClk = '1') THEN
                IF (IntRxEnaRam = '1') THEN
--                    RxDatOut <= IntRxDatOut;
                    RxDatOut(1) <= IntRxDatOut(1);
                    RxDatOut(3) <= IntRxDatOut(3);
                    RxDatOut(5) <= IntRxDatOut(5);
                    RxDatOut(7) <= IntRxDatOut(7);
                    RxDatOut(9) <= IntRxDatOut(9);
                    RxDatOut(11) <= IntRxDatOut(11);
                    --RxEnaRam <= tmp;
                    tmp      := NOT tmp;
                END IF;
                RxEnaRam <= tmp;
            END IF;
        END PROCESS;
    END GENERATE;

    FfAsRegNot : IF BramAsRegister = 0 AND FfAsRegister = 1 GENERATE
-- RLOC or LOC constraints must be set in the UCF file.
    BEGIN
        TheFfReg : PROCESS (RxClkNot, IntRxEnaRamN)
            VARIABLE tmp : STD_LOGIC;
        BEGIN
            IF (RxRst = '1') THEN
--                RxDatOut <= (OTHERS => '0');
                tmp      := '0';
            ELSIF (RxClkNot'EVENT AND RxClkNot = '1') THEN
                IF (IntRxEnaRamN = '1') THEN
--                    RxDatOut <= IntRxDatOut;
                    RxDatOut(0) <= IntRxDatOut(0);
                    RxDatOut(2) <= IntRxDatOut(2);
                    RxDatOut(4) <= IntRxDatOut(4);
--                    RxDatOut(5) <= IntRxDatOut(5);
                    RxDatOut(6) <= IntRxDatOut(6);
                    RxDatOut(8) <= IntRxDatOut(8);
                    RxDatOut(10) <= IntRxDatOut(10);
                    --RxEnaRam <= tmp;
                    tmp      := NOT tmp;
                END IF;
--                RxEnaRam <= tmp;
            END IF;
        END PROCESS;
    END GENERATE;

------------------------------------------------------------------------------------------------------------------
--
END AdcReceiver_arch;







