LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY SRAM IS
    PORT (
        CLK50            : IN    STD_LOGIC;
        CLK100           : IN    STD_LOGIC;
        CLK_RD           : IN    STD_LOGIC;
        RST              : IN    STD_LOGIC;
        -- setup
        do_daq         : IN    STD_LOGIC;
        daq_busy : OUT STD_LOGIC;
        DAQ_CDS          : IN    STD_LOGIC;
        IO_MODE          : IN    STD_LOGIC;
        do_clr_mem         : IN    STD_LOGIC;
        clr_mem_busy         : OUT   STD_LOGIC;
        ack_daq_error        : IN    STD_LOGIC;
        -- random read write
        got_io           : OUT   STD_LOGIC;
        io_wr            : IN    STD_LOGIC;
        io_addr          : IN    STD_LOGIC_VECTOR (17 DOWNTO 0);
        data_wr          : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
        data_rd          : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- adc interface
        first_pixel      : IN    STD_LOGIC;
        adc_data         : IN    STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- CDS interface
        CDS_first_pixel  : OUT   STD_LOGIC;
        CDS_data_valid   : OUT   STD_LOGIC;
        CDS_adc_samples  : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
        CDS_sram_samples : OUT   STD_LOGIC_VECTOR (143 DOWNTO 0);
        -- SDRAM chips
        RAM_A            : OUT   STD_LOGIC_VECTOR (17 DOWNTO 0);
        RAM_DQ           : INOUT STD_LOGIC_VECTOR (143 DOWNTO 0);
        RAM_ADV          : OUT   STD_LOGIC;
        RAM_CK_A         : OUT   STD_LOGIC;
        RAM_CK_B         : OUT   STD_LOGIC;
        RAM_E2           : OUT   STD_LOGIC;
        RAM_FT           : OUT   STD_LOGIC;
        RAM_G            : OUT   STD_LOGIC;
        RAM_LBO          : OUT   STD_LOGIC;
        RAM_RnW          : OUT   STD_LOGIC;
        RAM_PE           : OUT   STD_LOGIC;
        RAM_ZQ           : OUT   STD_LOGIC;
        -- TC
        TC               : OUT   STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END SRAM;

ARCHITECTURE SRAM_arch OF SRAM IS

    CONSTANT CDS_LATENCY : INTEGER := 4;

    SIGNAL write_sig    : STD_LOGIC;
    SIGNAL write_now    : STD_LOGIC;
    SIGNAL RAM_data_wr  : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL RAM_data_now : STD_LOGIC_VECTOR (143 DOWNTO 0);

    SIGNAL RAM_ADDR        : STD_LOGIC_VECTOR (17 DOWNTO 0);
    SIGNAL ram_data_in     : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL ram_data_in_not : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL ram_rw          : STD_LOGIC;

    TYPE   state_type IS (IDLE, ADDR_IO, WAIT_1ST, RD, WR, CDS_RD, CDS_WR, CDS_ACK, CLR_MEM_1ST, CLR_MEM, ERROR_1ST);
    SIGNAL state : state_type;

    SIGNAL cds_wr_data : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL cds_rd_data : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL addr0_sig   : STD_LOGIC;
    SIGNAL addr0_sig50 : STD_LOGIC;

    -- full frame cds support
    SIGNAL cds_data : STD_LOGIC_VECTOR (143 DOWNTO 0);
    SIGNAL first_pixel_delay : STD_LOGIC:='1';

    
BEGIN  -- SRAM_arch

    clr_mem_busy <= '1' WHEN state = CLR_MEM_1ST OR state = CLR_MEM ELSE '0';
    got_io   <= '1' WHEN state = ADDR_IO                        ELSE '0';
    daq_busy <= '1' WHEN state = WAIT_1ST OR state = RD OR state = WR ELSE '0';

    -- constant SRAM signals
    RAM_G   <= '0';
    RAM_ADV <= '0';
    RAM_E2  <= '1';
    RAM_ZQ  <= '1';
    RAM_PE  <= '0';
    RAM_LBO <= '0';
    RAM_FT  <= '0';

    RAM_CK_A <= NOT CLK100;
    RAM_CK_B <= NOT CLK100;

    --RAM_A <= RAM_ADDR;
    --RAM_RnW <= ram_rw;

    PROCESS (CLK100, RST)
        VARIABLE RAM_CDS_ADDR : STD_LOGIC_VECTOR (17 DOWNTO 0);
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            addr0_sig <= '0';
            write_sig <= '0';
            --RAM_RnW   <= '1';
            ram_rw    <= '1';
            state     <= IDLE;
        ELSIF CLK100'EVENT AND CLK100 = '1' THEN  -- rising clock edge
            RAM_A     <= RAM_ADDR;
            RAM_RnW   <= ram_rw;
            addr0_sig <= '0';
            CASE state IS
                WHEN IDLE =>
                    NULL;
                    -- reset WR
                    write_sig <= '0';
                    --RAM_RnW   <= '1';
                    ram_rw    <= '1';
                    -- addr 0
                    RAM_ADDR  <= (OTHERS => '0');
                    -- sram_d Z
						  first_pixel_delay<='1';
                    IF do_daq = '1' THEN
                        state <= WAIT_1ST;
                    ELSIF IO_MODE = '1' THEN
                        state <= ADDR_IO;
                    ELSIF do_clr_mem = '1' THEN
                        state <= CLR_MEM_1ST;
                    ELSE
                        state <= IDLE;
                    END IF;
                WHEN ADDR_IO =>
                    NULL;
                    -- set WR if wr
                    IF IO_WR = '1' THEN
                        write_sig <= '1';
                        --RAM_RnW   <= '0';
                        ram_rw    <= '0';
                    ELSE
                        write_sig <= '0';
                        --RAM_RnW   <= '1';
                        ram_rw    <= '1';
                    END IF;
                    -- addr input
                    RAM_ADDR    <= IO_addr;
                    -- sram_d data if wr
                    RAM_data_wr <= data_wr;
                    IF IO_MODE = '0' THEN
                        state <= IDLE;
                    ELSE
                        state <= ADDR_IO;
                    END IF;
                WHEN WAIT_1ST =>        -- wair for the first pixel marker
                    NULL;
                    -- reset WR
                    write_sig <= '0';
                    --RAM_RnW   <= '1';
                    ram_rw    <= '1';
                    -- addr 0
                    RAM_ADDR  <= (OTHERS => '0');
                    IF first_pixel = '1' THEN
                        state <= WR;
                    ELSE
                        IF do_daq = '1' THEN
                            state <= WAIT_1ST;
                        ELSE
                            -- exit path if no first pixel
                            state <= IDLE;
                        END IF;
                    END IF;
                WHEN RD =>              -- read sample DAQ mode
                    NULL;
                    -- set WR
                    write_sig <= '0';
                    --RAM_RnW   <= '1';
                    ram_rw    <= '1';
                    -- addr cnt
                    RAM_ADDR  <= RAM_ADDR + 1;
                    IF RAM_ADDR = (660 * 128) - 1 THEN
                        -- first pixel again
                        RAM_ADDR  <= (OTHERS => '0');
                        addr0_sig <= '1';
--                        IF first_pixel = '0' THEN
--                            state <= ERROR_1ST;
--                        ELS
		  first_pixel_delay<=first_pixel;
								
								IF DAQ_CDS = '1' THEN
                            ram_rw <= '1';  --'0';  -- don't write, no CDS data is
                                        -- available that soon
                            state  <= CDS_WR;
                        ELSIF do_daq = '1' THEN
                            ram_rw <= '1';  --'0';  -- don't write addr 0 with new frame
                            state  <= WR;
                        ELSE
                            state <= IDLE;
                        END IF;
                    ELSE
                        state <= WR;
                    END IF;
                WHEN WR =>              -- write sample DAQ mode
                    NULL;
                    -- reset WR
                    write_sig   <= '1';
                    --RAM_RnW   <= '0';
                    ram_rw      <= '0';
                    -- addr cnt
                    -- sram_d adc
                    RAM_data_wr <= adc_data;
                    --    IF do_daq = '1' THEN
						  if first_pixel_delay='0' then
						  state <= ERROR_1ST;
						  else
                    state       <= RD;
						  end if;
                    --    ELSE
                    --        state <= IDLE;
                    --    END IF;
                -- WHEN CDS_RD =>
                    -- ram_rw       <= '1';
                    -- write_sig    <= '0';
                    -- RAM_CDS_ADDR := RAM_CDS_ADDR + 1;
                    -- RAM_ADDR     <= RAM_CDS_ADDR;
                -- WHEN CDS_WR =>
                    -- RAM_ADDR    <= RAM_CDS_ADDR - CDS_LATENCY;
                    -- RAM_data_wr <= cds_data;
                    -- IF RAM_ADDR < CDS_LATENCY THEN
                        -- ram_rw    <= '1';
                        -- write_sig <= '0';
                        -- state     <= CDS_RD;
                    -- ELSIF RAM_ADDR = 660*128-1+CDS_LATENCY THEN
                        -- ram_rw    <= '1';
                        -- write_sig <= '0';
                        -- state     <= CDS_ACK;
                    -- ELSE
                        -- ram_rw    <= '0';
                        -- write_sig <= '1';
                        -- state     <= CDS_RD;
                    -- END IF;
                -- WHEN CDS_ACK =>
                    -- write_sig   <= '0';
                    -- ram_rw      <= '1';
                    -- RAM_data_wr <= (OTHERS => '0');
                    -- RAM_ADDR    <= (OTHERS => '0');
                    -- IF DAQ_CDS = '0' THEN
                        -- state <= IDLE;
                    -- END IF;
                WHEN CLR_MEM_1ST =>
                    write_sig   <= '1';
                    --RAM_RnW     <= '0';
                    ram_rw      <= '0';
                    RAM_data_wr <= (OTHERS => '0');
                    RAM_ADDR    <= (OTHERS => '0');
                    state       <= CLR_MEM;
                WHEN CLR_MEM =>         -- clear memory
                    NULL;
                    -- set WR 
                    write_sig   <= '1';
                    -- addr cnt
                    RAM_ADDR    <= RAM_ADDR + 1;
                    -- sram_d
                    RAM_data_wr <= (OTHERS => '0');
                    --RAM_data_wr (17 DOWNTO 0) <= RAM_ADDR;
                    IF RAM_ADDR = "11"&x"ffff" THEN
                        state <= IDLE;
                    ELSE
                        state <= CLR_MEM;
                    END IF;
                    --WHEN WAIT_DONE =>  -- wait for writes in the quene to finish
                    --    NULL;
                    --    -- reset WR
                    --    write_sig <= '0';
                    --    RnW       <= '1';
                    --    -- addr 0
                WHEN ERROR_1ST =>       -- error is 1st pixel marker not after
                                        -- 660*128 pixels
                    NULL;
                    -- reset WR
                    write_sig <= '0';
                    RAM_RnW   <= '1';
                    -- addr
                    RAM_ADDR  <= (OTHERS => '0');
                    IF ack_daq_error = '1' THEN
                        state <= IDLE;
                    ELSE
                        state <= ERROR_1ST;
                    END IF;
                WHEN OTHERS => NULL;
            END CASE;
        END IF;
    END PROCESS;

--    PROCESS (CLK100, RST)
--    BEGIN  -- PROCESS
--        IF RST = '0' THEN               -- asynchronous reset (active low)
--            --write_wait  <= '0';
--            write_now <= '0';
--            RAM_DQ    <= (OTHERS => 'Z');
--            --RAM_DATA_in <= (OTHERS => '0');
--        ELSIF CLK100'EVENT AND CLK100 = '1' THEN  -- rising clock edge
--            --write_wait  <= write_sig;
--            --write_now   <= write_wait;
--            write_now <= write_sig;
--            --RAM_DATA_in <= RAM_DQ;      
--
--            --RAM_data_wait <= RAM_data_wr;
--            --RAM_data_now  <= RAM_data_wait;
--            RAM_data_now <= RAM_data_wr;
--            -- IF write_now = '1' THEN
--            IF write_sig = '1' THEN
--                --RAM_DQ <= RAM_data_now;
--                RAM_DQ <= RAM_data_wr;
--            ELSE
--                RAM_DQ <= (OTHERS => 'Z');
--            END IF;
--        END IF;
--    END PROCESS;
    PROCESS (CLK100, RST)
        VARIABLE data_tmp      : STD_LOGIC_VECTOR (143 DOWNTO 0);
        VARIABLE write_sig_del : STD_LOGIC;
    BEGIN  -- PROCESS
        IF RST = '1' THEN
            RAM_DQ      <= (OTHERS => 'Z');
            cds_wr_data <= (OTHERS => '0');
        ELSIF CLK100'EVENT AND CLK100 = '1' THEN

            --IF write_sig = '1' THEN
            IF write_sig_del = '1' THEN
                --RAM_DQ <= RAM_data_wr;
                RAM_DQ      <= data_tmp;
					 IF state /= ADDR_IO THEN
					 	RAM_DQ(143 downto 128) <= (others=>'Z');
					 END IF;
                cds_wr_data <= data_tmp;
            ELSE
                RAM_DQ <= (OTHERS => 'Z');
            END IF;
            data_tmp      := RAM_data_wr;
            write_sig_del := write_sig;
        END IF;
    END PROCESS;


--    PROCESS (CLK100, RST)
--    BEGIN  -- PROCESS
--        IF RST = '0' THEN               -- asynchronous reset (active low)
--            ram_data_in_not <= (OTHERS => '0');
--        ELSIF CLK100'EVENT AND CLK100 = '0' THEN  -- rising clock edge
--            ram_data_in_not <= RAM_DQ;
--        END IF;
--    END PROCESS;

    PROCESS (CLK100, RST)
    --PROCESS (CLK_RD, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active low)
            ram_data_in <= (OTHERS => '0');
        ELSIF CLK100'EVENT AND CLK100 = '1' THEN  -- rising clock edge
        --ELSIF CLK_RD'EVENT AND CLK_RD = '1' THEN  -- rising clock edge
            ram_data_in <= RAM_DQ;

            --    IF ram_data_in_not = 0 THEN
            --        tc(0) <= '1';
            --    ELSE
            --        tc(0) <= '0';
            --    END IF;
        END IF;
    END PROCESS;
    tc(0) <= '1' WHEN state = ERROR_1ST ELSE '0';

--    PROCESS (CLK100, RST)
--    BEGIN  -- PROCESS
--        IF RST = '0' THEN               -- asynchronous reset (active low)
--
--        ELSIF CLK100'EVENT AND CLK100 = '1' THEN  -- rising clock edge
--            -- delay ADC to current sample output
--            -- register SRAM read data
--            SRAM_data_int <= SRAM_d;
--        END IF;
--    END PROCESS;

    -- last sample
--    last_sample <= SRAM_data_int;
    -- SRAM random read
    data_rd <= ram_data_in;

    -- purpose: store read data in DAQ mode
    PROCESS (CLK100, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active low)
            cds_rd_data <= (OTHERS => '0');
        ELSIF CLK100'EVENT AND CLK100 = '0' THEN  -- falling clock edge
            IF write_sig = '1' THEN
                cds_rd_data <= ram_data_in;
            END IF;
        END IF;
    END PROCESS;

    CDS_sram_samples <= cds_rd_data;
    CDS_adc_samples  <= cds_wr_data;

    -- sync and delay addr 0 (1st pixel) marker
    PROCESS (CLK100, RST)
        VARIABLE addr0 : STD_LOGIC;
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            addr0 := '0';
        ELSIF CLK100'EVENT AND CLK100 = '1' THEN  -- rising clock edge
            addr0_sig50 <= addr0 OR addr0_sig;
            addr0       := addr0_sig;
        END IF;
    END PROCESS;
    PROCESS (CLK50, RST)
        VARIABLE srg : STD_LOGIC_VECTOR (1 DOWNTO 0);
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)

        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
--            CDS_first_pixel <= srg(1);
--            srg(1)          := srg(0);
--            srg(0)          := addr0_sig50;
            CDS_first_pixel <= addr0_sig50;
        END IF;
    END PROCESS;

    -- delay ADC data and do CDS for full frame CDS
--    PROCESS (CLK100, RST)
--        VARIABLE cds_rd_data100 : STD_LOGIC_VECTOR (143 DOWNTO 0);
--    BEGIN  -- PROCESS
--        IF RST = '1' THEN               -- asynchronous reset (active high)
--            cds_data       <= (OTHERS => '0');
--            cds_rd_data100 := (OTHERS => '0');
--        ELSIF CLK100'EVENT AND CLK100 = '1' THEN  -- rising clock edge
--            cds_data <= (OTHERS => '0');
--            FOR i IN 0 TO 7 LOOP
--                cds_data(i*16+11 DOWNTO i*16) <= cds_rd_data100(i*16+11 DOWNTO i*16) - adc_data(i*16+11 DOWNTO i*16);
--            END LOOP;  -- i
            -- cds_data <= cds_rd_data - adc_data;
            -- preserve Hot pixel MAP
--            cds_data(143 DOWNTO 136) <= cds_rd_data100(143 DOWNTO 136);
            -- register at 100MHzclock to ease timing
--            cds_rd_data100           := cds_rd_data;
--        END IF;
--    END PROCESS;

END SRAM_arch;





