-----------------------------------------------------------------------------------------------------------------------------------------
-- Entity Name:  Toplevel_V2_Smpl_Nff
-- File Name:    Toplevel_V2_Smpl_Nff.vhd
-- File Path:    Projects\Ads527x\Vhdl\Virtex2V1000Platform\AdsV2SmplNoFifo
-- Project : 
--
-- Purpose:     Provide a interface to the TMS ADS5273 Analog to digital converter.
--              to a Xilinx Virtex-II device (2V1000-6Fg456).
--              SPECIALLY adapted for the Texas Instruments Xc2V1000 Test Platform
--              Only one receiver channel per BRAM and in each receiver module 
--              a timing module.
--
--              In this toplevel of the design several parameter, through generics in 
--              VHDL code, can be selected.
--                ExternalClockIn 
--                  An external system clock is used for the design.
--                  When '1' external clock is used and a DCM is provided. When '0' no
--                  external clock is provided and the DCM is removed.
--                  When using a spearate system clock DCM, as in most cases, don't
--                  forget to eneable the attributes for the DCM.
--
--                BramAsRegister
--                  FfAsRegister
--                    These two settings determine the use of FFs, BRAM as register of BRAM as
--                    FF in the design. FfAsRegister puts normal FF's as first parallel stage 
--                    after the ADC interface, while BramAsRegister uses a BRAM as parallel
--                    register (one fixed address is used to store and read from).
--                      FfAsRegister  BramAsRegister                                            
--                      0               0               No register at all, direct out. Meaning a FIFO can be connected.
--                      0               1               Block SelectRAM used as register.
--                      1               0               Normal FFs register (look out for extra BUFG's).
--                      1               1               DO NOT USE !
--
--                              
--              All source code files contain extensive guidelines on possible "option" adaptations.
--
-- Revision History:
--                      Created:        28/04/04
--                      Modified:       22/06/04 Output registers removed from IOB.
--                                                                      Timing of RxEnaRam & RxClkRam shorter.
--                                                                      IOB contraint added to the UCF file (because to difficult in VHDL code)
--                                                                      Synplicity introduces BUFG components for every clock net to the registers.
--                                                                      Only way to get rid of is a Synplicity .sdc file with constraints.
--                                                                      Other solution is the use of a BRAM as simple register.
--                                                                      Implemented this possibility.
--                                              13/07/04 Addition of ADCLK polarity select.
--
----------------------------------------------------------------------------------------------------------------------------------------
--


LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_UNSIGNED.ALL;
USE IEEE.std_logic_arith.ALL;
-- Simulation libraries
LIBRARY UNISIM;
USE UNISIM.ALL;

ENTITY adc IS
    GENERIC (
--=============================================================================================================
        BramAsRegister : INTEGER := 1;
        FfAsRegister   : INTEGER := 0
        --  FfAsRegister  BramAsRegister                                            
        --  0             0               No register at all, direct out.
        --  0             1               Block SelectRAM used as register.
        --  1             0               Normal FFs register (look out for extra BUFG's).
        --  1             1               DO NOT USE !
--==============================================================================================================
        );
    PORT (
        -- FPGA interface
        CLK50      : IN  STD_LOGIC;
        CLK100     : IN  STD_LOGIC;
        RST        : IN  STD_LOGIC;
        DCM_locked : OUT STD_LOGIC;
        -- ADS interface
        ADCLK      : OUT STD_LOGIC;
        Ads5273Rst : OUT STD_LOGIC;
        LclkP      : IN  STD_LOGIC;     -- Six times ADCLK
        LclkN      : IN  STD_LOGIC;
        AdClkP     : IN  STD_LOGIC;  -- Skewed and Data aligned version of the ADC sample clock. Used as strobe.
        AdClkN     : IN  STD_LOGIC;
        DatInCh0P  : IN  STD_LOGIC;
        DatInCh0N  : IN  STD_LOGIC;
        DatInCh1P  : IN  STD_LOGIC;
        DatInCh1N  : IN  STD_LOGIC;
        DatInCh2P  : IN  STD_LOGIC;
        DatInCh2N  : IN  STD_LOGIC;
        DatInCh3P  : IN  STD_LOGIC;
        DatInCh3N  : IN  STD_LOGIC;
        DatInCh4P  : IN  STD_LOGIC;
        DatInCh4N  : IN  STD_LOGIC;
        DatInCh5P  : IN  STD_LOGIC;
        DatInCh5N  : IN  STD_LOGIC;
        DatInCh6P  : IN  STD_LOGIC;
        DatInCh6N  : IN  STD_LOGIC;
        DatInCh7P  : IN  STD_LOGIC;
        DatInCh7N  : IN  STD_LOGIC;
        -- ADS SPI interface
        -- data out
        adc_data0  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        adc_data1  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        adc_data2  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        adc_data3  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        adc_data4  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        adc_data5  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        adc_data6  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        adc_data7  : OUT STD_LOGIC_VECTOR (11 DOWNTO 0);
        -- Test Connector
        TC         : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END adc;

ARCHITECTURE adc_arch OF adc IS
----------------------------------------------------------------------------------------------------------------------------------------
-- Component Instantiation
----------------------------------------------------------------------------------------------------------------------------------------
-- FF primitive with async preset
    COMPONENT FDP PORT (C             : IN STD_LOGIC; D : IN STD_LOGIC; PRE : IN STD_LOGIC; Q : OUT STD_LOGIC); END COMPONENT;
-- FF primitive with async clear
    COMPONENT FDC PORT (C             : IN STD_LOGIC; D : IN STD_LOGIC; CLR : IN STD_LOGIC; Q : OUT STD_LOGIC); END COMPONENT;
-- CMOS25 input buffer primitive
    COMPONENT IBUF_LVCMOS33 PORT (I   : IN STD_LOGIC; O : OUT STD_LOGIC); END COMPONENT;
-- Single ended clock input buffer primitive
    COMPONENT IBUFG_LVCMOS33 PORT (I  : IN STD_LOGIC; O : OUT STD_LOGIC); END COMPONENT;
-- CMOS25 output buffer primitive
    COMPONENT OBUF_LVCMOS33 PORT (I   : IN STD_LOGIC; O : OUT STD_LOGIC); END COMPONENT;
-- Lvds input buffer primyive
    COMPONENT IBUFDS_LVDS_33 PORT (I  : IN STD_LOGIC; IB : IN STD_LOGIC; O : OUT STD_LOGIC); END COMPONENT;
-- Lvds clock input buffer primitive
    COMPONENT IBUFGDS_LVDS_33 PORT (I : IN STD_LOGIC; IB : IN STD_LOGIC; O : OUT STD_LOGIC); END COMPONENT;
-- BUFG primitive
    COMPONENT BUFG PORT (I            : IN STD_LOGIC; O : OUT STD_LOGIC); END COMPONENT;
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
    COMPONENT AdcReceiver
        GENERIC (
            BramAsRegister : INTEGER;  -- When 1, a BRAM will be used as register in this source.
            FfAsRegister   : INTEGER  -- Take the value given at the Entity level of this file.               
            );
        PORT (
            RxClk        : IN  STD_LOGIC;
            RxClkNot     : IN  STD_LOGIC;
            RxRst        : IN  STD_LOGIC;
            RxDatIn      : IN  STD_LOGIC;
            RxSyncIn     : IN  STD_LOGIC;
            PreSyncInP   : IN  STD_LOGIC;  -- Sync coming from previous module. if not used tie it to GND.
            PreSyncInN   : IN  STD_LOGIC;  -- Sync coming from previous module. if not used tie it to GND.
            PostSyncOutP : OUT STD_LOGIC;  -- Sync to the next module.
            PostSyncOutN : OUT STD_LOGIC;  -- Sync to the next module.
            AdClkOut     : OUT STD_LOGIC;  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     : OUT STD_LOGIC;  -- Used with an external BRAM fifo.
            RxDatOut     : OUT STD_LOGIC_VECTOR (11 DOWNTO 0)
            );
    END COMPONENT;
--------------------------------------------------------------------------------
-- To solve AdClk timing problems
    COMPONENT FD PORT (D, C : IN STD_LOGIC; Q : OUT STD_LOGIC); END COMPONENT;
    --COMPONENT FDC PORT (D, C, CLR : IN STD_LOGIC; Q : OUT STD_LOGIC); END COMPONENT;
--------------------------------------------------------------------------------
    COMPONENT adc_dcm
        PORT (
            CLKIN_N_IN        : IN  STD_LOGIC;
            CLKIN_P_IN        : IN  STD_LOGIC;
            RST_IN            : IN  STD_LOGIC;
            CLKDV_OUT         : OUT STD_LOGIC;
            CLKIN_IBUFGDS_OUT : OUT STD_LOGIC;
            CLK0_OUT          : OUT STD_LOGIC;
            CLK180_OUT        : OUT STD_LOGIC;
            LOCKED_OUT        : OUT STD_LOGIC
            );
    END COMPONENT;

    SIGNAL CLK_ChipScope : STD_LOGIC;

----------------------------------------------------------------------------------------------------------------------------------------
-- Signal, constant and variable Declarations
----------------------------------------------------------------------------------------------------------------------------------------
    CONSTANT Low  : STD_LOGIC := '0';
    CONSTANT High : STD_LOGIC := '1';

    SIGNAL IntRxClk    : STD_LOGIC;
    SIGNAL IntRxClkNot : STD_LOGIC;

    SIGNAL IntReceiverRst : STD_LOGIC;
    SIGNAL IntAds5273Rst  : STD_LOGIC;

    SIGNAL IntAdClk  : STD_LOGIC;
    SIGNAL IntDatCh0 : STD_LOGIC;
    SIGNAL IntDatCh1 : STD_LOGIC;
    SIGNAL IntDatCh2 : STD_LOGIC;
    SIGNAL IntDatCh3 : STD_LOGIC;
    SIGNAL IntDatCh4 : STD_LOGIC;
    SIGNAL IntDatCh5 : STD_LOGIC;
    SIGNAL IntDatCh6 : STD_LOGIC;
    SIGNAL IntDatCh7 : STD_LOGIC;

    SIGNAL IntAdClkOut7 : STD_LOGIC;
    SIGNAL IntAdClkOut6 : STD_LOGIC;
    SIGNAL IntAdClkOut5 : STD_LOGIC;
    SIGNAL IntAdClkOut4 : STD_LOGIC;
    SIGNAL IntAdClkOut3 : STD_LOGIC;
    SIGNAL IntAdClkOut2 : STD_LOGIC;
    SIGNAL IntAdClkOut1 : STD_LOGIC;
    SIGNAL IntAdClkOut0 : STD_LOGIC;

    SIGNAL IntFrameFrom6To7P : STD_LOGIC;
    SIGNAL IntFrameFrom5To6P : STD_LOGIC;
    SIGNAL IntFrameFrom4To5P : STD_LOGIC;
    SIGNAL IntFrameFrom3To4P : STD_LOGIC;
    SIGNAL IntFrameFrom2To3P : STD_LOGIC;
    SIGNAL IntFrameFrom1To2P : STD_LOGIC;
    SIGNAL IntFrameFrom0To1P : STD_LOGIC;
    SIGNAL IntFrameFrom6To7N : STD_LOGIC;
    SIGNAL IntFrameFrom5To6N : STD_LOGIC;
    SIGNAL IntFrameFrom4To5N : STD_LOGIC;
    SIGNAL IntFrameFrom3To4N : STD_LOGIC;
    SIGNAL IntFrameFrom2To3N : STD_LOGIC;
    SIGNAL IntFrameFrom1To2N : STD_LOGIC;
    SIGNAL IntFrameFrom0To1N : STD_LOGIC;

-- To accomodate for Sync/Frame timing
    SIGNAL IntAdClkP          : STD_LOGIC;
    SIGNAL IntAdClkOneP       : STD_LOGIC;
    SIGNAL IntAdClkOneNotP    : STD_LOGIC;
    SIGNAL IntTopPostSyncOutP : STD_LOGIC;
    SIGNAL IntAdClkN          : STD_LOGIC;
    SIGNAL IntAdClkOneN       : STD_LOGIC;
    SIGNAL IntAdClkOneNotN    : STD_LOGIC;
    SIGNAL IntTopPostSyncOutN : STD_LOGIC;
    SIGNAL DCMRESET : STD_LOGIC:='0';
    SIGNAL DCMLOCK : STD_LOGIC:='0';
    SIGNAL DCMCOUNTER : STD_LOGIC_VECTOR (25 DOWNTO 0):=(others=>'0');

    ATTRIBUTE RLOC_ORIGIN           : STRING;
-- RLOC positions for a XC2V1000-6FG456
	attribute RLOC_ORIGIN of AdcCh7	: label is "X28Y75";	-- Place the receiver module for channel 7
	attribute RLOC_ORIGIN of AdcCh6	: label is "X24Y75";	-- Place the receiver module for channel 6
	attribute RLOC_ORIGIN of AdcCh5	: label is "X20Y75";	-- Place the receiver module for channel 5
	attribute RLOC_ORIGIN of AdcCh4	: label is "X16Y75";	-- Place the receiver module for channel 4
	attribute RLOC_ORIGIN of AdcCh3	: label is "X12Y75";	-- Place the receiver module for channel 3
	attribute RLOC_ORIGIN of AdcCh2	: label is "X8Y75";	-- Place the receiver module for channel 2
	attribute RLOC_ORIGIN of AdcCh1	: label is "X4Y75";	-- Place the receiver module for channel 1
	attribute RLOC_ORIGIN of AdcCh0	: label is "X0Y75";	-- Place the receiver module for channel 0

    
-- To solve AdClk frame timing problems
    ATTRIBUTE IOB                   : STRING;
    ATTRIBUTE IOB OF TopFfIobSp     : LABEL IS "TRUE";
    ATTRIBUTE IOB OF TopFfIobSn     : LABEL IS "TRUE";

----------------------------------------------------------------------------------------------------------------------------------------
    signal     IntDatRec0 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    signal     IntDatRec1 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    signal     IntDatRec2 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    signal     IntDatRec3 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    signal     IntDatRec4 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    signal     IntDatRec5 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    signal     IntDatRec6 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    signal     IntDatRec7 :  STD_LOGIC_VECTOR (11 DOWNTO 0);
    
    --CONSTANT BramAsRegister : INTEGER := 1;
    --CONSTANT FfAsRegister   : INTEGER := 0;

 SIGNAL RxEnaRam : STD_LOGIC_VECTOR (7 DOWNTO 0);

    --SIGNAL DCM_locked : STD_LOGIC;
    
--
BEGIN


------------------------------------------------------------------------
------------------------------------------------------------------------
------------------------------------------------------------------------
-- Clock generation, RX Clock manager and circuitry
------------------------------------------------------------------------
--
    inst_adc_dcm : adc_dcm
        PORT MAP (
            CLKIN_N_IN        => LclkN,
            CLKIN_P_IN        => LclkP,
            RST_IN            => DCMRESET,
            CLKDV_OUT         => CLK_ChipScope,
            CLKIN_IBUFGDS_OUT => OPEN,
  --          CLK0_OUT          => IntRxClk,
  --          CLK180_OUT        => IntRxClkNot,
	         CLK0_OUT          => IntRxClkNot,
            CLK180_OUT        => IntRxClk,
            LOCKED_OUT        => DCMLOCK
            );
			DCM_locked<=DCMLOCK;
	      PROCESS (CLK50)
    BEGIN  -- PROCESS
--        IF RST = '1' or (DCMLOCK='0' and CONV_INTEGER(DCMCOUNTER)=1000000) THEN               -- asynchronous reset (active high)
        IF RST = '1'  THEN               -- asynchronous reset (active high)
			  DCMCOUNTER<=(others=>'0');
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN
		   if CONV_INTEGER(DCMCOUNTER)<100000 then
			  DCMCOUNTER<=DCMCOUNTER+1;
			  if CONV_INTEGER(DCMCOUNTER)<50000 then
			  DCMRESET<='1';
			  else
			  DCMRESET<='0';
			  end if;
			end if;

--			if DCMLOCK='0' then
--			  DCMCOUNTER<=(others=>'0');
--			end if;
		  END IF;
		  END PROCESS;

--------------------------------------------------------------------------------

--------------------------------------------------------------------------------
-- ADS5273 receiver interface and FIFO
-- For this application the FIFO's are omitted, no backk-end design is foreseen
-- and everything is replaced with a simple register.
-- Even this register can be omitted. Then the output of the parallel load register
-- is directly connected to the IOB.
--------------------------------------------------------------------------------
-- Reset signal for the ADS5273 Analog-Digital Converter.
-- The ADC's reset is active LOW (while the FPPGA's reset are active high).        CLK50        : IN  STD_LOGIC;

    AdcRstFf    : FDC PORT MAP (C            => IntRxClk, D => High, CLR => IntReceiverRst, Q => IntAds5273Rst);
    --AdcRstIob   : OBUF_LVCMOS33 PORT MAP (I  => IntAds5273Rst, O => Ads5273Rst);
----------------------------------------------------------------
-- Frame or Sync input
    SyncIob     : IBUFDS_LVDS_33 PORT MAP (I => AdClkP, IB => AdClkN, O => IntAdClk);
--
-- This is needed to accomodate for the timing at high speed.
-- The AdClk input pads are placed to far from the timing circuit for the first input block.
-- Therefore it was not possible to meet the timing between the first FF place in the AdClk IOB
-- and the second FF. 
-- Solution is to provide a AdClk forwarding circuit as used in each timing block for control
-- of the inter timing block framing signals.
-- On this toplevel an IOB FF an inverter and a FF are introduced betwen the first timing block
-- and the input pads of AdClk. The firsttiming block input RxSyncIn is not used, but in stead the 
-- PreSyncIn inputs are used.
--              Added stuff:
--                      In the component decaration section : an FD primitive and FDC primitive
--                      In the signals and attributes section : an IOB LOC constraint to use the IOB FFs.
--                                                                                                      signal declarations.
--
    TopFfIobSp  : FD PORT MAP (D             => IntAdClk, C => IntRxClk, Q => IntAdcLkP);
    TopFfOneP   : FDC PORT MAP (D            => IntAdClkP, C => IntRxClk, CLR => IntReceiverRst, Q => IntAdClkOneP);
    IntAdClkOneNotP <= NOT IntAdClkOneP;
    TopFfPostSp : FDC PORT MAP (D            => IntAdClkOneNotP, C => IntRxClk, CLR => IntReceiverRst, Q => IntTopPostSyncOutP);
--
    TopFfIobSn  : FD PORT MAP (D             => IntAdClk, C => IntRxClkNot, Q => IntAdcLkN);
    TopFfOneN   : FDC PORT MAP (D            => IntAdClkN, C => IntRxClkNot, CLR => IntReceiverRst, Q => IntAdClkOneN);
    IntAdClkOneNotN <= NOT IntAdClkOneN;
    TopFfPostSn : FDC PORT MAP (D            => IntAdClkOneNotN, C => IntRxClkNot, CLR => IntReceiverRst, Q => IntTopPostSyncOutN);
--
-- The TopPostSyncOut(P/N) signals are connected with the Channel 0 PreSyncIn inputs as if a extra channel
-- was placed in front.
-- Extra inserts are used in the UCF file to meet timing.
----------------------------------------------------------------
-- Channel 7
--
    Ch7Iob      : IBUFDS_LVDS_33 PORT MAP (I => DatInCh7P, IB => DatInCh7N, O => IntDatCh7);

    AdcCh7 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh7,
            RxSyncIn     => Low,        -- Input of the external sync
            PreSyncInP   => IntFrameFrom6To7P,  -- input from lower module.
            PreSyncInN   => IntFrameFrom6To7N,  -- input from lower module.
            PostSyncOutP => OPEN,       -- output to upper module.
            PostSyncOutN => OPEN,       -- output to upper module.
            AdClkOut     => IntAdClkOut7,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(7),
            RxDatOut     => IntDatRec7
            );

-- Channel 6
--
    Ch6Iob : IBUFDS_LVDS_33 PORT MAP (I => DatInCh6P, IB => DatInCh6N, O => IntDatCh6);

    AdcCh6 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh6,
            RxSyncIn     => Low,        -- Input of the external sync
            PreSyncInP   => IntFrameFrom5To6P,  -- input from lower module.
            PreSyncInN   => IntFrameFrom5To6N,  -- input from lower module.
            PostSyncOutP => IntFrameFrom6To7P,  -- output to upper module.
            PostSyncOutN => IntFrameFrom6To7N,  -- output to upper module.
            AdClkOut     => IntAdClkOut6,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(6),
            RxDatOut     => IntDatRec6
            );

-- Channel 5
--
    Ch5Iob : IBUFDS_LVDS_33 PORT MAP (I => DatInCh5P, IB => DatInCh5N, O => IntDatCh5);

    AdcCh5 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh5,
            RxSyncIn     => Low,  -- Input of the external sync, high holds the FF in reset (Q = zero)
            PreSyncInP   => IntFrameFrom4To5P,  -- input from lower module
            PreSyncInN   => IntFrameFrom4To5N,  -- input from lower module
            PostSyncOutP => IntFrameFrom5To6P,  -- output to upper module.
            PostSyncOutN => IntFrameFrom5To6N,  -- output to upper module.
            AdClkOut     => IntAdClkOut5,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(5),
            RxDatOut     => IntDatRec5
            );

-- Channel 4
--
    Ch4Iob : IBUFDS_LVDS_33 PORT MAP (I => DatInCh4P, IB => DatInCh4N, O => IntDatCh4);

    AdcCh4 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh4,
            RxSyncIn     => Low,  -- Input of the external sync, high holds the FF in reset (Q = zero)
            PreSyncInP   => IntFrameFrom3To4P,  -- input from lower module
            PreSyncInN   => IntFrameFrom3To4N,  -- input from lower module
            PostSyncOutP => IntFrameFrom4To5P,  -- output to upper module.
            PostSyncOutN => IntFrameFrom4To5N,  -- output to upper module.
            AdClkOut     => IntAdClkOut4,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(4),
            RxDatOut     => IntDatRec4
            );

-- Channel 3
--
    Ch3Iob : IBUFDS_LVDS_33 PORT MAP (I => DatInCh3P, IB => DatInCh3N, O => IntDatCh3);

    AdcCh3 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh3,
            RxSyncIn     => Low,  -- Input of the external sync, high holds the FF in reset (Q = zero)
            PreSyncInP   => IntFrameFrom2To3P,  -- input from lower module
            PreSyncInN   => IntFrameFrom2To3N,  -- input from lower module
            PostSyncOutP => IntFrameFrom3To4P,  -- output to upper module.
            PostSyncOutN => IntFrameFrom3To4N,  -- output to upper module.
            AdClkOut     => IntAdClkOut3,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(3),
            RxDatOut     => IntDatRec3
            );

-- Channel 2
--
    Ch2Iob : IBUFDS_LVDS_33 PORT MAP (I => DatInCh2P, IB => DatInCh2N, O => IntDatCh2);

    AdcCh2 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh2,
            RxSyncIn     => Low,  -- Input of the external sync, high holds the FF in reset (Q = zero)
            PreSyncInP   => IntFrameFrom1To2P,  -- input from lower module
            PreSyncInN   => IntFrameFrom1To2N,  -- input from lower module
            PostSyncOutP => IntFrameFrom2To3P,  -- output to upper module.
            PostSyncOutN => IntFrameFrom2To3N,  -- output to upper module.
            AdClkOut     => IntAdClkOut2,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(2),
            RxDatOut     => IntDatRec2
            );

-- Channel 1
--
    Ch1Iob : IBUFDS_LVDS_33 PORT MAP (I => DatInCh1P, IB => DatInCh1N, O => IntDatCh1);

    AdcCh1 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh1,
            RxSyncIn     => Low,  -- Input of the external sync, high holds the FF in reset (Q = zero)
            PreSyncInP   => IntFrameFrom0To1P,  -- input from lower module
            PreSyncInN   => IntFrameFrom0To1N,  -- input from lower module
            PostSyncOutP => IntFrameFrom1To2P,  -- output to upper module.
            PostSyncOutN => IntFrameFrom1To2N,  -- output to upper module.
            AdClkOut     => IntAdClkOut1,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(1),
            RxDatOut     => IntDatRec1
            );

-- Channel 0
--
    Ch0Iob : IBUFDS_LVDS_33 PORT MAP (I => DatInCh0P, IB => DatInCh0N, O => IntDatCh0);

    AdcCh0 : AdcReceiver
        GENERIC MAP (
            BramAsRegister => BramAsRegister,  -- Take the value given at the Entity level of this file.
            FfAsRegister   => FfAsRegister  -- Take the value given at the Entity level of this file.
            )
        PORT MAP (
            RxClk        => IntRxClk,
            RxClkNot     => IntRxClkNot,
            RxRst        => IntReceiverRst,
            RxDatIn      => IntDatCh0,
            RxSyncIn     => Low,  -- Input of the external sync, high holds the FF in reset (Q = zero)
            -- To accomodate for timing the RxSyncIn input is not directly used.
            -- A small circuit is placed in from and provides signals to the PreSyncIn Inputs as if a 
            -- Timing block was placed in front of it.
            PreSyncInP   => IntTopPostSyncOutP,  -- input from lower module
            PreSyncInN   => IntTopPostSyncOutN,  -- input from lower module
            PostSyncOutP => IntFrameFrom0To1P,   -- output to upper module.
            PostSyncOutN => IntFrameFrom0To1N,   -- output to upper module.
            AdClkOut     => IntAdClkOut0,  -- One Lclk later than PostSyncOut, for signal analyser purposes
            RxEnaRam     => RxEnaRam(0),
            RxDatOut     => IntDatRec0
            );

--

    IntReceiverRst <= RST;
    AdClk          <= CLK50;



    PROCESS (CLK50, RST)
	 --PROCESS (IntRxClk, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            adc_data0 <= (OTHERS=>'0');
            adc_data1 <= (OTHERS=>'0');
            adc_data2 <= (OTHERS=>'0');
            adc_data3 <= (OTHERS=>'0');
            adc_data4 <= (OTHERS=>'0');
            adc_data5 <= (OTHERS=>'0');
            adc_data6 <= (OTHERS=>'0');
            adc_data7 <= (OTHERS=>'0');
        ELSIF CLK50'event AND CLK50 = '0' THEN  -- rising clock edge
		  --ELSIF IntRxClk'event AND IntRxClk = '1' THEN  -- rising clock edge
            adc_data0 <= IntDatRec0;
            adc_data1 <= IntDatRec1;
            adc_data2 <= IntDatRec2;
            adc_data3 <= IntDatRec3;
            adc_data4 <= IntDatRec4;
            adc_data5 <= IntDatRec5;
            adc_data6 <= IntDatRec6;
            adc_data7 <= IntDatRec7;
        END IF;
    END PROCESS;

    TC(7 DOWNTO 1) <= (OTHERS => '0');
    -- TC(1) <= '1' WHEN RxEnaRam=x"00" ELSe '0';
    TC(0) <= CLK_ChipScope;


    ADC_Reset_Test: PROCESS (CLK50, RST)
        VARIABLE cnt : INTEGER;
    BEGIN  -- PROCESS ADC_Reset_Test	RxEnaRam <= IntRxEnaRam;
        IF RST = '1' THEN               -- asynchronous reset (active high)
            cnt := 0;
				Ads5273Rst <='1';
        ELSIF CLK50'event AND CLK50 = '1' THEN  -- rising clock edge
            IF cnt < 300000 THEN
                Ads5273Rst <= '1';
            ELSIF cnt < 800000 THEN
                Ads5273Rst <='0';
            ELSE
                Ads5273Rst <= '1';
            END IF;
            IF cnt < 1000000 THEN
                cnt := cnt + 1;
            END IF;
        END IF;
    END PROCESS ADC_Reset_Test;
END adc_arch;
