LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_arith.ALL;
USE IEEE.std_logic_unsigned.ALL;


ENTITY linebuffer IS
    PORT (
        CLK50           : IN  STD_LOGIC;
        RST             : IN  STD_LOGIC;
        -- setup
        no_marker       : IN  STD_LOGIC;
        -- data IN
        first_pixel_in  : IN  STD_LOGIC;
        cds_data        : IN  STD_LOGIC_VECTOR (8 DOWNTO 0);
        -- data out
        first_pixel_out : OUT STD_LOGIC;
        data_valid      : OUT STD_LOGIC;
        pixel_addr      : OUT STD_LOGIC_VECTOR (16 DOWNTO 0);
        data_line0_9_o  : OUT STD_LOGIC_VECTOR (8 DOWNTO 0);
        data_line1_9_o  : OUT STD_LOGIC_VECTOR (8 DOWNTO 0);
        data_line2_9_o  : OUT STD_LOGIC_VECTOR (8 DOWNTO 0);
        -- test connector
        TC              : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );	  
END linebuffer;   

ARCHITECTURE linebuffer_arch OF linebuffer IS

    CONSTANT nMARKERS : INTEGER := 20;
    CONSTANT nFRAMES  : INTEGER := 10;
    CONSTANT nPIXELS  : INTEGER := 66;

    COMPONENT sorter_dpm
        PORT (
            addra : IN  STD_LOGIC_VECTOR(10 DOWNTO 0);
            addrb : IN  STD_LOGIC_VECTOR(10 DOWNTO 0);
            clka  : IN  STD_LOGIC;
            clkb  : IN  STD_LOGIC;
            dina  : IN  STD_LOGIC_VECTOR(8 DOWNTO 0);
            doutb : OUT STD_LOGIC_VECTOR(8 DOWNTO 0);
            wea   : IN  STD_LOGIC
            );
    END COMPONENT;

    COMPONENT linebuffer_dpm
        PORT (
            addra : IN  STD_LOGIC_VECTOR(9 DOWNTO 0);
            addrb : IN  STD_LOGIC_VECTOR(9 DOWNTO 0);
            clka  : IN  STD_LOGIC;
            clkb  : IN  STD_LOGIC;
            dina  : IN  STD_LOGIC_VECTOR(17 DOWNTO 0);
            doutb : OUT STD_LOGIC_VECTOR(17 DOWNTO 0);
            wea   : IN  STD_LOGIC
            );
    END COMPONENT;

    SIGNAL frame_cnt : INTEGER RANGE 0 TO 9;
    SIGNAL pixel_cnt : INTEGER RANGE 0 TO 65;
    SIGNAL line_cnt  : INTEGER RANGE 0 TO 128;

    SIGNAL sector_select : STD_LOGIC := '0';

    SIGNAL linesorter_wr_addr : STD_LOGIC_VECTOR (9 DOWNTO 0);
    SIGNAL linebuffer_rd_addr : STD_LOGIC_VECTOR (9 DOWNTO 0);
    SIGNAL linebuffer_wr_addr : STD_LOGIC_VECTOR (9 DOWNTO 0);

    SIGNAL linesorter_addr_wr : STD_LOGIC_VECTOR (10 DOWNTO 0);
    SIGNAL linesorter_addr_rd : STD_LOGIC_VECTOR (10 DOWNTO 0);
    SIGNAL linbuffer_data_wr  : STD_LOGIC_VECTOR (17 DOWNTO 0);
    SIGNAL linbuffer_data_rd  : STD_LOGIC_VECTOR (17 DOWNTO 0);

    SIGNAL data_sorter_out : STD_LOGIC_VECTOR (8 DOWNTO 0);
    SIGNAL line1_out       : STD_LOGIC_VECTOR (8 DOWNTO 0);
    SIGNAL line2_out       : STD_LOGIC_VECTOR (8 DOWNTO 0);

    SIGNAL first_pixel_last : STD_LOGIC;
    
BEGIN  -- linebuffer_arch



    -- pixel sequence from teh mimostar2
    -- mimostart 2 has 10 subframes
    -- chip reads 1 pixel per subframe and the takes the next pixel
    pixel_sorter : PROCESS (CLK50, RST)
    BEGIN  -- PROCESS pixel_sorter
        IF RST = '1' THEN               -- asynchronous reset (active high)
            frame_cnt <= 0;
            pixel_cnt <= 0;
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF first_pixel_in = '1' THEN
                frame_cnt     <= 0;
                pixel_cnt     <= 0;
                line_cnt      <= 0;
                sector_select <= NOT sector_select;
            ELSE
                IF frame_cnt = nFRAMES - 1 AND pixel_cnt = nPIXELS -1 THEN
                    frame_cnt     <= 0;
                    pixel_cnt     <= 0;
                    line_cnt      <= line_cnt + 1;
                    sector_select <= NOT sector_select;
                ELSIF frame_cnt = nFRAMES - 1 THEN
                    frame_cnt <= 0;
                    pixel_cnt <= pixel_cnt + 1;
                ELSE
                    frame_cnt <= frame_cnt + 1;
                END IF;
            END IF;
        END IF;
    END PROCESS pixel_sorter;
    linesorter_wr_addr <= conv_std_logic_vector(frame_cnt * nPIXELS + pixel_cnt, 10);

    pixel_addr(9 DOWNTO 0)   <= linesorter_wr_addr;
    pixel_addr(16 DOWNTO 10) <= conv_std_logic_vector(line_cnt, 7);

    --linebuffer_wr_addr <= (linebuffer_rd_addr - 1) WHEN (linebuffer_rd_addr - 1)(9 DOWNTO 0) | = "11" ELSE 660;

    second_third_line : PROCESS (CLK50, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            linebuffer_rd_addr <= (OTHERS => '0');
            first_pixel_last <= '0';
            first_pixel_out <= '0';
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF linebuffer_rd_addr = (nFRAMES * nPIXELS)-1 OR first_pixel_in = '1' THEN
                linebuffer_rd_addr <= (OTHERS => '0');
                first_pixel_out <= first_pixel_last;
                first_pixel_last <= first_pixel_in;
            ELSE
                linebuffer_rd_addr <= linebuffer_rd_addr + 1;
                first_pixel_out <= '0';
            END IF;
            linebuffer_wr_addr <= linebuffer_rd_addr;
            IF no_marker = '1' AND linebuffer_rd_addr < nMARKERS THEN
                data_valid <= '0';
            ELSE
                data_valid <= '1';
            END IF;
        END IF;
    END PROCESS;

    linesorter_addr_wr <= (sector_select & linesorter_wr_addr);
    linesorter_addr_rd <= ((NOT sector_select) & linebuffer_rd_addr);
    -- sorter memory
    -- dual ported
    -- 9 bits wide
    -- 2k deep
    -- A write port
    -- B read port
    sorter_dpm_inst : sorter_dpm
        PORT MAP (
            addra => linesorter_addr_wr,  --(sector_select & linesorter_wr_addr),
            addrb => linesorter_addr_rd,  --((NOT sector_select) & linebuffer_rd_addr),
            clka  => CLK50,
            clkb  => CLK50,
            dina  => cds_data,
            doutb => data_sorter_out,
            wea   => '1'
            );

    linbuffer_data_wr <= data_sorter_out & line1_out;
    line1_out         <= linbuffer_data_rd(17 DOWNTO 9);
    line2_out         <= linbuffer_data_rd(8 DOWNTO 0);
    data_line0_9_o    <= data_sorter_out;
    data_line1_9_o    <= line1_out;
    data_line2_9_o    <= line2_out;

    -- linebuffer memory
    -- dual ported
    -- 18 bits wide
    -- 1k deep
    -- A write port
    -- B read port
    linebuffer_dpm_inst : linebuffer_dpm
        PORT MAP (
            addra => linebuffer_wr_addr,
            addrb => linebuffer_rd_addr,
            clka  => CLK50,
            clkb  => CLK50,
            dina  => linbuffer_data_wr,  --data_sorter_out & line1_out,
            doutb => linbuffer_data_rd,  --line1_out & line2_out,
            wea   => '1'
            );


END linebuffer_arch;




