LIBRARY IEEE;
USE IEEE.std_logic_1164.ALL;
USE IEEE.std_logic_unsigned.ALL;
USE IEEE.std_logic_arith.ALL;

ENTITY minostar2 IS
    PORT (
        CLK50       : IN  STD_LOGIC;
        CLK100      : IN  STD_LOGIC;
        RST         : IN  STD_LOGIC;
        -- setup
        fake_chip   : IN  STD_LOGIC := '0';
        resync      : IN  STD_LOGIC;    -- when doing jtag
        -- mimostar2
        CkRdC       : OUT STD_LOGIC;
        SYNC        : OUT STD_LOGIC;
        MxFirst     : IN  STD_LOGIC;
        -- to memory
        first_pixel : OUT STD_LOGIC;
        frame_err   : OUT STD_LOGIC;
        -- test connector
        TC          : OUT STD_LOGIC_VECTOR (7 DOWNTO 0)
        );
END minostar2;

ARCHITECTURE minostar2_arch OF minostar2 IS

    TYPE   state_type IS (IDLE, SEND_SYNC, FIRST_PIXEL_STATE, WAIT_FIRST_PIXEL);
    SIGNAL state : state_type;

    SIGNAL pixel_cnt    : INTEGER RANGE 0 TO 660*128;
    SIGNAL MxFirst_sync : STD_LOGIC;

BEGIN  -- minostar2_arch

    PROCESS (CLK50, RST)
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            pixel_cnt <= 0;
            frame_err <= '0';
            state     <= IDLE;
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            frame_err <= '0';
            CASE state IS
                WHEN IDLE =>
                    state <= SEND_SYNC;
                WHEN SEND_SYNC =>
                    pixel_cnt <= (660 * 128)-1;
                    IF MxFirst_sync = '1' OR fake_chip = '1' THEN
                        state <= FIRST_PIXEL_STATE;
                    END IF;
                WHEN FIRST_PIXEL_STATE =>
                    IF pixel_cnt /= (660 * 128) - 1 THEN
                        frame_err <= '1';
                    END IF;
                    pixel_cnt <= 0;
                    state     <= WAIT_FIRST_PIXEL;
                    IF resync = '1' THEN
                        state <= SEND_SYNC;
                    END IF;
                WHEN WAIT_FIRST_PIXEL =>
                    pixel_cnt <= pixel_cnt + 1;
                    IF (MxFirst_sync = '1' AND fake_chip = '0') OR (fake_chip = '1' AND pixel_cnt = (660*128)-2) THEN
                        state <= FIRST_PIXEL_STATE;
                    END IF;
                    IF resync = '1' THEN
                        state <= SEND_SYNC;
                    END IF;
                WHEN OTHERS =>
                    NULL;
            END CASE;
        END IF;
    END PROCESS;

    first_pixel <= '1' WHEN state = FIRST_PIXEL_STATE ELSE '0';
    PROCESS (CLK50, RST)
        VARIABLE sync_cnt : INTEGER RANGE 0 TO 15;
    BEGIN  -- PROCESS CLK50
        IF RST = '1' THEN               -- asynchronous reset (active high)
            SYNC <= '0';
        ELSIF CLK50'EVENT AND CLK50 = '1' THEN  -- rising clock edge
            IF state = SEND_SYNC THEN
                IF sync_cnt = 15 THEN
                    SYNC <= '0';
                ELSE
                    sync_cnt := sync_cnt + 1;
                    SYNC     <= '1';
                END IF;
            ELSE
                sync_cnt := 0;
                SYNC <= '0';
            END IF;
        END IF;
    END PROCESS;
    --  SYNC <= '1' WHEN state = SEND_SYNC ELSE '0';

    PROCESS (CLK100, RST)
        VARIABLE clock : STD_LOGIC;
    BEGIN  -- PROCESS
        IF RST = '1' THEN               -- asynchronous reset (active high)
            --CkRdC <= '1';
            --clock := '1';
				CkRdC <= '1';				  -- michal 12_Apr_2007 to align signals with ADC clock
            clock := '1';
        ELSIF CLK100'EVENT AND CLK100 = '1' THEN  -- rising clock edge
		  --ELSIF CLK100'EVENT AND CLK100 = '0' THEN  -- michal 12_Apr_2007 to align signals with ADC clock
            CkRdC <= clock;
            clock := NOT clock;
        END IF;
    END PROCESS;

    sync_mxfirst : PROCESS (CLK50, RST)
        VARIABLE sync_srg : STD_LOGIC_VECTOR (5 DOWNTO 0);
    BEGIN  -- PROCESS sync_mxfirst
        IF RST = '1' THEN               -- asynchronous reset (active high)
            sync_srg := (OTHERS => '0');
        ELSIF CLK50'EVENT AND CLK50 = '0' THEN  -- rising clock edge
            --IF sync_srg(3 DOWNTO 0) = "1100" THEN
            IF sync_srg(3 DOWNTO 0) = "1100" THEN 
                MxFirst_sync <= '1';
            ELSE
                MxFirst_sync <= '0';
            END IF;
            sync_srg(4 DOWNTO 0) := sync_srg(5 DOWNTO 1);
            sync_srg(5)          := MxFirst;
        END IF;
    END PROCESS sync_mxfirst;

END minostar2_arch;






